/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/****  Project : eXtended C Library (XCLIB)
    *  Purpose : GENERIC LIST
    *  Language: ANSI C
    *  File    : demo_list-1.c
    *  Author  : Martin Vicente <martin@vicente.org>
    *
    ****/


/*****************************************************************************\

 [English]

 Purely generic use of <xc/list>.

 [French]

 Utilisation purement gnrique de <xc/list>.

\*****************************************************************************/


#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <xc/xalloc.h>
#include <xc/list.h>

static void demo_numbers(void);
static void demo_strings(void);

static int put_number(Id id, void *data);
static int cmp_number(Id id1, Id id2, void *data);
static int tst_number(Id id, void *data);

static int put_string(Id id, void *data);
static int cmp_string(Id id1, Id id2, void *data);
static int tst_string(Id id, void *data);

#define COMM(en, fr)	__xcint(comm)(en, fr, NULL)
#define WAIT()		__xcint(wait)()

extern void __xcint(comm)(const char *en, ...);	/* DON'T USE IT! */
extern void __xcint(wait)(void);		/* DON'T USE IT! */

int main()
{
    demo_numbers();
    demo_strings();

    return 0;
}

static void demo_numbers(void)
{
    List list;
    Id   id;

    int one = 1, two = 2, three = 3;

    COMM("Demonstration with numbers",
	 "Demonstration avec des nombres");

    list = LIST.create(0);

    LIST.append(list, &three);
    LIST.append(list, &one);
    LIST.append(list, &two);

    LIST.action(list, put_number, NULL);

    WAIT();

    COMM("Sorting numbers",
	 "Tri des nombres");

    LIST.sort(list, cmp_number, NULL);

    LIST.action(list, put_number, NULL);

    WAIT();

    COMM("Searching an number",
	 "Recherche d'un nombre");

    if ((id = LIST.search(list, tst_number, (void *)2)) != NULL)
    {
	printf("\tFound %d\n", *(const int *)LIST.item(id));
	printf("\tFound %d\n", *(const int *)id->item);

	COMM("Modify an element of the list",
	     "Modification d'un lment de la liste");

	*(int *)LIST.item(id) = 7;
	*(int *)id->item      = 7;
    }
    else
    {
	puts("Not Found");
    }

    LIST.action(list, put_number, NULL);

    WAIT();

    COMM("Creation of a table of pointers on list's elements",
	 "Production d'une table de pointeurs sur les lments de la liste");

    {
	const int **const table = LIST.table(list, 0);
	size_t      const count = LIST.count(list);

	register size_t i;

	for (i = 0; i < count; ++i) printf("\t*table[%d] = %d\n", i, *table[i]);

	xfree(table);
    }

    WAIT();

    COMM("Creation of a table containing a copy of each list's element",
	 "Production d'une table contenant une copie de chaque lment");

    {
	int   *const table = LIST.table(list, sizeof (int));
	size_t const count = LIST.count(list);

	register size_t i;

	for (i = 0; i < count; ++i) printf("\ttable[%d] = %d\n", i, table[i]);

	for (i = 0; i < count; ++i) table[i] *= 2;

	putchar('\n');

	for (i = 0; i < count; ++i) printf("\ttable[%d] = %d\n", i, table[i]);

	xfree(table);
    }

    putchar('\n');

    LIST.action(list, put_number, NULL);

    WAIT();

    LIST.destroy(list);
}

static void demo_strings(void)
{
    List list;
    Id   id;

    COMM("Demonstration with strings",
	 "Demonstration avec des chanes");

    list = LIST.create(0);

    LIST.append(list, "ccc");
    LIST.append(list, "a");
    LIST.append(list, "bb");

    LIST.action(list, put_string, NULL);

    WAIT();

    COMM("Sorting strings",
	 "Tri des chanes");

    LIST.sort(list, cmp_string, NULL);

    LIST.action(list, put_string, NULL);

    WAIT();

    COMM("Searching an string",
	 "Recherche d'une chane");

    if ((id = LIST.search(list, tst_string, "bb")) != NULL)
    {
	printf("\tFound %s\n", (const char *)LIST.item(id));
	printf("\tFound %s\n", (const char *)id->item);

	COMM("Modify an element of the list",
	     "Modification d'un lment de la liste");

#if defined __GNUC__ && !defined __STRICT_ANSI__
	(const char *)id->item = "Hello";
#else
	id->item = "Hello";
#endif
    }
    else
    {
	puts("Not Found");
    }

    LIST.action(list, put_string, NULL);

    WAIT();

    COMM("Creation of a table of pointers on list's elements",
	 "Production d'une table de pointeurs sur les lments de la liste");

    {
	const char **const table = LIST.table(list, 0);
	size_t       const count = LIST.count(list);

	register size_t i;

	for (i = 0; i < count; ++i) printf("\ttable[%d] = %s\n", i, table[i]);

	xfree(table);
    }

    WAIT();

    COMM("Creation of a table containing a copy of each list's element",
	 "Production d'une table contenant une copie de chaque lment");

    {
	char  *const table = LIST.table(list, sizeof (char));
	size_t const count = LIST.count(list);

	register size_t i;

	for (i = 0; i < count; ++i) printf("\ttable[%d] = %c\n", i, table[i]);

	xfree(table);
    }

    WAIT();

    LIST.destroy(list);
}

static int put_number(Id id, __unused void *data)
{
    printf("\tnumber: %d\n", *(const int *)id->item);

    return 0;
}

static int cmp_number(Id id1, Id id2, __unused void *data)
{
    /* Sorting numbers in increasing order */

    return *(const int *)id1->item - *(const int *)id2->item;
}

static int tst_number(Id id, void *data)
{
    return *(const int *)id->item == (int)data;
}

static int put_string(Id id, __unused void *data)
{
    printf("\tstring: %s\n", (const char *)id->item);

    return 0;
}

static int cmp_string(Id id1, Id id2, __unused void *data)
{
    /* Sorting strings in lexicographical order */

    return strcmp(id1->item, id2->item);
}

static int tst_string(Id id, void *data)
{
    return strcmp(id->item, data) == 0;
}


/* End of file */
