/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/****  Project : eXtended C Library (XCLIB)
    *  Purpose : GENERIC LIST
    *  Language: ANSI C
    *  File    : demo_list-5.c
    *  Author  : Martin Vicente <martin@vicente.org>
    *
    ****/


/*****************************************************************************\

 [English]

 <xc/list> use with type checking.

 Demonstration of the hard disk writing of an object list, its reading,
 in binary mode. Caution: The objects structure, here, has to be entirely
 known by the module <xc/list>. Therefore, it has to be completely defined
 statically.

 [French]

 Utilisation de <xc/list> avec contrle de type.

 Dmonstration de l'criture sur le disque d'une liste d'objets, et de sa
 lecture, en mode binaire. Attention: la structure des objets doit ici tre
 entirement connue du module <xc/list>. La structure doit donc tre
 intgralement dfinie statiquement.

\*****************************************************************************/


#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

#include <xc/strlib.h>
#include <xc/error.h>
#include <xc/xalloc.h>
#include <xc/list.h>

typedef struct ObjectRec *Object;

def_LIST(OBJECTS,, Object);

#define OBJECTS	(*OBJECTS)	/* optional */

#define FILENAME	"objects.bin"

struct ObjectRec
{
    int  number;
    char name[8];
};

static Object createObject(const char *name);
static void   destroyObject(Object object);

static int print_object(ObjectId id, void *data);
static int destroy_object(ObjectId id, void *data);

#define COMM(en, fr)	__xcint(comm)(en, fr, NULL)
#define WAIT()		__xcint(wait)()

extern void __xcint(comm)(const char *en, ...);	/* DON'T USE IT! */
extern void __xcint(wait)(void);		/* DON'T USE IT! */

int main()
{
    ObjectList list;
    FILE      *file;

    COMM("Creating list of objects",
	 "Cration de la liste d'objets");

    list = OBJECTS.create(sizeof (struct ObjectRec));

    OBJECTS.add(list, createObject("Black"));
    OBJECTS.add(list, createObject("Red"));
    OBJECTS.add(list, createObject("Green"));
    OBJECTS.add(list, createObject("Yellow"));
    OBJECTS.add(list, createObject("Blue"));
    OBJECTS.add(list, createObject("Magenta"));
    OBJECTS.add(list, createObject("Cyan"));
    OBJECTS.add(list, createObject("White"));

    OBJECTS.action(list, print_object, NULL);

    WAIT();

    COMM("Writing to disk",
	 "Ecriture sur disque de la liste d'objets");

    if ((file = fopen(FILENAME, "w")) == NULL)
	error("Can't open file");

    if (OBJECTS.write(list, NULL, NULL, file) == EOF)
	error("Can't write file");

    if (fclose(file) == EOF)
	error("Can't close file");

    COMM("Destroying object's list",
	 "Destruction de la liste d'objets");

    OBJECTS.action(list, destroy_object, NULL);
    OBJECTS.destroy(list);

    WAIT();

    COMM("Reading list from disk",
	 "Cration de la liste d'objets depuis le fichier disque");

    if ((file = fopen(FILENAME, "r")) == NULL)
	error("Can't open file");

    if ((list = OBJECTS.read(NULL, NULL, file)) == NULL)
	error("Can't read file");

    if (fclose(file) == EOF)
	error("Can't close file");

    OBJECTS.action(list, print_object, NULL);

    WAIT();

    COMM("Destroying object's list",
	 "Destruction de la liste d'objets");

    OBJECTS.action(list, destroy_object, NULL);
    OBJECTS.destroy(list);

    return 0;
}

static Object createObject(const char *name)
{
    static int number = 0;

    Object object;

    assert(name != NULL);

    object = xnew(Object);

    object->number = ++number;

    strcpy(object->name, name);

    return object;
}

static void destroyObject(Object object)
{
    assert(object != NULL);

    xfree(object);
}

static int print_object(ObjectId id, __unused void *data)
{
    printf("\tObject %d - %s\n", id->item->number, id->item->name);

    return 0;
}

static int destroy_object(ObjectId id, __unused void *data)
{
    destroyObject(id->item);

    return 0;
}


/* End of file */
