/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : PRIMARY FUNCTIONS
  | Version : 0.3
  | Language: ANSI C
  | File    : base/text2exit.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


#define _XCLIB_INTERNALS

#include <stddef.h>
#include <string.h>
#include <stdlib.h>

#include <xc/errors.h>

#include "xc/base.h"

#include "global.h"

#define EQn(txt, err)	(strncmp(txt, err, strlen(err)) == 0)

int __xclib(text2exit)(const char *const text)
{
#if 0
    int status = EXIT_FAILURE;

    if (text != NULL && text[0] != '\0')
    {
	if      (text ==   err_success)     status = EXIT_SUCCESS;
	else if (text ==   err_failure)     status = EXIT_FAILURE;
	else if (text ==   err_memfull)     status = EXIT_MEMFULL;
	else if (text ==   err_sysfail)     status = EXIT_SYSFAIL;
	else if (EQn(text, err_success))    status = EXIT_SUCCESS;
	else if (EQn(text, err_failure))    status = EXIT_FAILURE;
	else if (EQn(text, err_memfull))    status = EXIT_MEMFULL;
	else if (EQn(text, err_sysfail))    status = EXIT_SYSFAIL;
	/**/
	else if (EQn(text, ERR_SUCCESS))    status = EXIT_SUCCESS;
	else if (EQn(text, ERR_FAILURE))    status = EXIT_FAILURE;
	else if (EQn(text, ERR_MEMFULL))    status = EXIT_MEMFULL;
	else if (EQn(text, ERR_SYSFAIL))    status = EXIT_SYSFAIL;
	else if (EQn(text, ERR_SYSCALL))    status = EXIT_SYSCALL;
	else if (EQn(text, ERR_CLIBERR))    status = EXIT_CLIBERR;
	else if (EQn(text, ERR_NORANGE))    status = EXIT_NORANGE;
	else if (EQn(text, ERR_CONVERT))    status = EXIT_CONVERT;
	else if (EQn(text, ERR_IOERROR))    status = EXIT_IOERROR;
	else if (EQn(text, ERR_NETWORK))    status = EXIT_NETWORK;
	else if (EQn(text, ERR_DISPLAY))    status = EXIT_DISPLAY;
	else if (EQn(text, ERR_XWINDOW))    status = EXIT_XWINDOW;
	else if (EQn(text, ERR_INVALID))    status = EXIT_INVALID;
	/**/
	else if (EQn(text, ERR_INVOCATION)) status = EXIT_INVALID;
	else if (EQn(text, ERR_INVOPT))     status = EXIT_INVALID;
	else if (EQn(text, ERR_NOOPTARG))   status = EXIT_INVALID;
	else if (EQn(text, ERR_INVOPTARG))  status = EXIT_INVALID;
    }

    return status;
#else
    int status;

    status = get_exit_status(text);

    if (status == 0) status = EXIT_FAILURE;

    return status;
#endif
}

#include "startup.h"
{
    set_exit_status(ERR_MEMFULL, EXIT_MEMFULL);
    set_exit_status(ERR_SYSFAIL, EXIT_SYSFAIL);
    set_exit_status(ERR_SYSCALL, EXIT_SYSCALL);
    set_exit_status(ERR_CLIBERR, EXIT_CLIBERR);
    set_exit_status(ERR_NORANGE, EXIT_NORANGE);
    set_exit_status(ERR_CONVERT, EXIT_CONVERT);
    set_exit_status(ERR_IOERROR, EXIT_IOERROR);
    set_exit_status(ERR_NETWORK, EXIT_NETWORK);
    set_exit_status(ERR_DISPLAY, EXIT_DISPLAY);
    set_exit_status(ERR_XWINDOW, EXIT_XWINDOW);
    set_exit_status(ERR_INVALID, EXIT_INVALID);
    set_exit_status(ERR_PROGRAM, EXIT_PROGRAM);

    set_exit_status(ERR_INVOCATION, EXIT_INVALID);
    set_exit_status(ERR_INVOPT,     EXIT_INVALID);
    set_exit_status(ERR_INVOPTARG,  EXIT_INVALID);
    set_exit_status(ERR_INTERNAL,   EXIT_PROGRAM);
    set_exit_status(ERR_CASSERT,    EXIT_PROGRAM);
    set_exit_status(ERR_MAXARGS,    EXIT_PROGRAM);
}

#include "cleanup.h"
{
}


/* End Of File */
