/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : EXTRA MATHEMATIC FUNCTIONS
  | Version : 0.3
  | Language: ANSI C
  | File    : math/add.h
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


/*****************************************************************************\

Rappel:

  EDOM    Math argument out of domain of function
  ERANGE  Math result not representable

Attention:

  L'algorithme utilis ici pour la version "best" n'est pas encore la meilleur
  que l'on puisse envisager. Il sera propos dans une version future.

\*****************************************************************************/


#define CARD	unsigned int
#define L_MAX	UINT_MAX
#define U_MIN	0

#if !defined INTEGER && !defined FLOATING_POINT ||\
     defined INTEGER &&  defined FLOATING_POINT
#error "xclib: *** INTEGER or FLOATING_POINT!"
#endif

static TYPE fast(TYPE table[], CARD count);
static TYPE best(TYPE table[], CARD count);

TYPE FNAME(TYPE table[], CARD count)
{
    return (_math_flags & _mathFast ? fast : best)(table, count);
}

static TYPE fast(TYPE table[], CARD count)
{
    TYPE result = 0;

    register CARD i;

    assert(table != NULL && count > 0);

    for (i = 0; i < count; ++i)
    {
	TYPE const x = table[i];

	if (!(_math_flags & _mathNoRangeChecking))
	{
	    if (result > 0)
	    {
		if (x > 0 && x > TYPE_MAX - result)
		{
		    errno = ERANGE;

		    return TYPE_MAX; /* overflow */
		}
	    }
	    else if (result < 0)
	    {
		if (x < 0 && x < TYPE_MIN - result)
		{
		    errno = ERANGE;

		    return TYPE_MIN; /* underflow */
		}
	    }
	}

	result += x;
    }

    if (result == TYPE_MAX || result == TYPE_MIN) errno = 0;

    return result;
}

typedef int (*compar_fn_t)(const void *, const void *);

static int qcmp(TYPE *x1, TYPE *x2);

static TYPE best(TYPE table[], CARD count)
{
    TYPE result;

    assert(table != NULL && count > 0);

    /* Trie des valeurs en ordre croissant */
    qsort(table, count, sizeof table[0], (compar_fn_t)qcmp);

    assert(TYPE_MIN < 0 && TYPE_MAX > 0);

    result = 0;

    {
	register CARD l = 0;
	register CARD u = count - 1;

	while (l <= u)
	{
	    if (result > 0)
	    {
		TYPE const x = table[l];

		if (!(_math_flags & _mathNoRangeChecking))
		{
		    if (x > 0 && x > TYPE_MAX - result)
		    {
			errno = ERANGE;

			return TYPE_MAX; /* overflow */
		    }
		}

		result += x;

		if (l == L_MAX) break;

		l++;
	    }
	    else if (result < 0)
	    {
		TYPE const x = table[u];

		if (!(_math_flags & _mathNoRangeChecking))
		{
		    if (x < 0 && x < TYPE_MIN - result)
		    {
			errno = ERANGE;

			return TYPE_MIN; /* underflow */
		    }
		}

		result += x;

		if (u == U_MIN) break;

		u--;
	    }
#if 0
	   /* Aprs une seconde rflexion, la partie conditionnelle qui suit
	    * est superflue. Le rsultat du calcul de l'utilisateur en flottant
	    * ne doit de toute manire pas tenir compte de cette diffrence de
	    * comportement - grce  des tests d'encadrement - qui est de plus
	    * trs peu frquent (`result' devant tre gal  zro). */	      
#ifdef FLOATING_POINT
	   /* Facultatif: offre une symtrie de comportement pour les cas
	    * limites de dpassements (voir test_add). */
	    else if (Abs(table[u]) > Abs(table[l]))
	    {
		result = table[u];

		if (u == U_MIN) break;

		u--;
	    }
#endif
#endif
	    else /* result == 0 */
	    {
		result = table[l];

		if (l == L_MAX) break;

		l++;
	    }
	}
    }

    if (result == TYPE_MAX || result == TYPE_MIN) errno = 0;

    return result;
}

static int qcmp(TYPE *x1, TYPE *x2)
{
#if 1
   /* Avec des flottants, la comparaison est nettement plus rapide.
    * Pas de diffrence notable avec des entiers.
    */ 
    return *x1 > *x2 ? +1 : *x1 < *x2 ? -1 : 0;
#else
    return *x1 - *x2;
#endif
}


/* END OF FILE */
