/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : PROTECTED MEMORY ALLOCATION FUNCTIONS
  | Version : 0.3
  | Language: ANSI C
  | File    : xalloc/xrealloc.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


#define _XCLIB_INTERNALS

#define _FACILITY FACILITY(xalloc)

#include <stddef.h>
#include <stdlib.h>
#include <assert.h>

#include <xc/errors.h>
#include <xc/error.h>

#include "xc/xalloc.h"

#include "global.h"

#include "xalloc/errors.h"

void *xrealloc(void *ptr, size_t const size)
{
    assert(ptr != NULL || size > 0);

    if (ptr == NULL && size > 0)
    {
	ptr = malloc(size);

	if (ptr == NULL) error(ERR_MALLOC);

	return ptr;
    }
    else if (ptr != NULL && size == 0)
    {
	free(ptr);

	return NULL;
    }
    else
    {
	ptr = realloc(ptr, size);

	if (ptr == NULL) error(ERR_REALLOC);

	return ptr;
    }
}

#include "startup.h"
{
    set_exit_status(ERR_MALLOC,  EXIT_MEMFULL);
    set_exit_status(ERR_REALLOC, EXIT_MEMFULL);
}

#include "cleanup.h"
{
    set_exit_status(ERR_MALLOC,  0);
    set_exit_status(ERR_REALLOC, 0);
}


/* End Of File */
