############################################################## -*- makefile -*-
#
#	Copyright (C) 1996-2001  Martin Vicente
#	This file is part of the eXtended C Library (XCLIB).
#
#	This library is free software; you can redistribute it and/or
#	modify it under the terms of the GNU Library General Public
#	License as published by the Free Software Foundation; either
#	version 2 of the License, or (at your option) any later version.
#
#	This library is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#	Library General Public License for more details.
#
#	You should have received a copy of the GNU Library General Public
#	License along with this library (see the file COPYING.LIB).
#	If not, write to the Free Software Foundation, Inc.,
#	59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################
#
#	DISPLAY COLORIZATION
#
# Warning: This is an include file for GNU makefiles only.
#
#	Project : "Les outils de Martin"
#	Purpose : Colors definition for GNU makefile
#	File    : utils/color.mk
#	Version : 1.1
#	Language: GNU Make
#	Author  : Martin Vicente
#	Contact : martin@vicente.org (or martin.vicente@free.fr)
#	Revision: Mon, 14 May 2001
#
# Description:
#
#   Escape sequences definition based on the ECMA-48/ISO, 6429/ANSI, and X3.64
#   "Set Graphics Rendition".
#
# Parameters:
#
#   COLOR    = No/Yes/COLOR-NAME
#   BG_COLOR = No/Yes/COLOR-NAME
#
#	COLOR-NAME is one of:
#	Black/Red/Green/Yellow/Blue/Magenta/Cyan/White
#
###############################################################################

ifneq (,)
This makefile requires GNU Make.
endif

ifndef   __UTILS_COLOR_MK
override __UTILS_COLOR_MK := OK

#==============================================================================

override UTILS ?= $(HOME)/devel/utils

include $(UTILS)/utils.mk

#====( DEFINITIONS )

ifdef COLOR
  ifeq ($(COLOR), 0)
    override COLOR :=
  else
    override COLOR := $(shell echo $(COLOR) | $(LOWER))
  ifeq ($(COLOR), false)
    override COLOR :=
  else
  ifeq ($(COLOR), no)
    override COLOR :=
  else
  ifeq ($(COLOR), off)
    override COLOR :=
  endif
  endif
  endif
  endif
endif

ifdef COLOR

  override esc := '\033'

  override save    := $(esc)7
  override restore := $(esc)8

  override reset := $(esc)[0m

  override bold          := $(esc)[1m
  override bold_off      := $(esc)[22m
  override underline     := $(esc)[4m
  override underline_off := $(esc)[24m
  override blink         := $(esc)[5m
  override blink_off     := $(esc)[25m
  override reverse       := $(esc)[7m
  override reverse_off   := $(esc)[27m

  override Bold      = $(bold)$(strip $1)$(bold_off)
  override Underline = $(underline)$(strip $1)$(underline_off)
  override Blink     = $(blink)$(strip $1)$(blink_off)
  override Reverse   = $(reverse)$(strip $1)$(reverse_off)

  override black   := $(esc)[30m
  override red     := $(esc)[31m
  override green   := $(esc)[32m
  override yellow  := $(esc)[33m
  override blue    := $(esc)[34m
  override magenta := $(esc)[35m
  override cyan    := $(esc)[36m
  override white   := $(esc)[37m
  override default := $(esc)[39m

  override Black   = $(black)$(strip $1)$(default)
  override Red     = $(red)$(strip $1)$(default)
  override Green   = $(green)$(strip $1)$(default)
  override Yellow  = $(yellow)$(strip $1)$(default)
  override Blue    = $(blue)$(strip $1)$(default)
  override Magenta = $(magenta)$(strip $1)$(default)
  override Cyan    = $(cyan)$(strip $1)$(default)
  override White   = $(white)$(strip $1)$(default)

  override color := $($(COLOR))

  ifndef color

    override color := $(default)

  else

    ifdef BG_COLOR
      ifeq ($(BG_COLOR), 0)
        override BG_COLOR :=
      else
        override BG_COLOR := $(shell echo $(BG_COLOR) | $(LOWER))
      ifeq ($(BG_COLOR), false)
        override BG_COLOR :=
      else
      ifeq ($(BG_COLOR), no)
        override BG_COLOR :=
      else
      ifeq ($(BG_COLOR), off)
        override BG_COLOR :=
      endif
      endif
      endif
      endif
    endif

    ifdef BG_COLOR

      override bg_black   := $(esc)[40m
      override bg_red     := $(esc)[41m
      override bg_green   := $(esc)[42m
      override bg_yellow  := $(esc)[43m
      override bg_blue    := $(esc)[44m
      override bg_magenta := $(esc)[45m
      override bg_cyan    := $(esc)[46m
      override bg_white   := $(esc)[47m
      override bg_default := $(esc)[49m

      override BgBlack   = $(bg_black)$(strip $1)$(bg_default)
      override BgRed     = $(bg_red)$(strip $1)$(bg_default)
      override BgGreen   = $(bg_green)$(strip $1)$(bg_default)
      override BgYellow  = $(bg_yellow)$(strip $1)$(bg_default)
      override BgBlue    = $(bg_blue)$(strip $1)$(bg_default)
      override BgMagenta = $(bg_magenta)$(strip $1)$(bg_default)
      override BgCyan    = $(bg_cyan)$(strip $1)$(bg_default)
      override BgWhite   = $(bg_white)$(strip $1)$(bg_default)

      override bg_color := $(bg_$(BG_COLOR))

      ifndef bg_color

        override color    := $(default)
        override bg_color := $(bg_default)

      endif

      override BgColor = $(bg_color)$(strip $1)$(bg_default)

    endif

  endif

  override Color = $(color)$(strip $1)$(default)

endif

override Bold      ?= $(strip $1)
override Underline ?= $(strip $1)
override Blink     ?= $(strip $1)
override Reverse   ?= $(strip $1)

override Black     ?= $(strip $1)
override Red       ?= $(strip $1)
override Green     ?= $(strip $1)
override Yellow    ?= $(strip $1)
override Blue      ?= $(strip $1)
override Magenta   ?= $(strip $1)
override Cyan      ?= $(strip $1)
override White     ?= $(strip $1)
override Color     ?= $(strip $1)

override BgBlack   ?= $(strip $1)
override BgRed     ?= $(strip $1)
override BgGreen   ?= $(strip $1)
override BgYellow  ?= $(strip $1)
override BgBlue    ?= $(strip $1)
override BgMagenta ?= $(strip $1)
override BgCyan    ?= $(strip $1)
override BgWhite   ?= $(strip $1)
override BgColor   ?= $(strip $1)

#==============================================================================

endif  # __UTILS_COLOR_MK

# [END OF FILE]
